@@ -4,6 +4,12 @@
 #include <vector>
 #include <cstring>
 
+#include <ft2build.h>
+#include FT_FREETYPE_H
+#include FT_TRUETYPE_TABLES_H
+
+#include "utils.h"
+
 enum FontWeight {
   FontWeightUndefined   = 0,
   FontWeightThin        = 100,
@@ -30,6 +36,24 @@ enum FontWidth {
   FontWidthUltraExpanded  = 9
 };
 
+inline FontWeight get_font_weight(FT_Face face) {
+  void* table = FT_Get_Sfnt_Table(face, FT_SFNT_OS2);
+  if (table == NULL) {
+    return FontWeightUndefined;
+  }
+  TT_OS2* os2_table = (TT_OS2*) table;
+  return (FontWeight) os2_table->usWeightClass;
+}
+
+inline FontWidth get_font_width(FT_Face face) {
+  void* table = FT_Get_Sfnt_Table(face, FT_SFNT_OS2);
+  if (table == NULL) {
+    return FontWidthUndefined;
+  }
+  TT_OS2* os2_table = (TT_OS2*) table;
+  return (FontWidth) os2_table->usWidthClass;
+}
+
 struct FontDescriptor {
 public:
   const char *path;
@@ -66,7 +90,7 @@ public:
     this->italic = italic;
     this->monospace = false;
   }
-  
+
   // Constructor added by Thomas Lin Pedersen
   FontDescriptor(const char *family, bool italic, FontWeight weight, FontWidth width) {
     this->path = NULL;
@@ -80,6 +104,19 @@ public:
     this->monospace = false;
   }
 
+  // Constructor added by Thomas Lin Pedersen
+  FontDescriptor(FT_Face face, const char* path, int index) {
+    this->path = copyString(path);
+    this->index = index;
+    this->postscriptName = FT_Get_Postscript_Name(face) == NULL ? "" : FT_Get_Postscript_Name(face);
+    this->family = copyString(face->family_name);
+    this->style = copyString(face->style_name);
+    this->weight = get_font_weight(face);
+    this->width = get_font_width(face);
+    this->italic = face->style_flags & FT_STYLE_FLAG_ITALIC;
+    this->monospace = FT_IS_FIXED_WIDTH(face);
+  }
+
   FontDescriptor(const char *path, const char *postscriptName, const char *family, const char *style,
                  FontWeight weight, FontWidth width, bool italic, bool monospace) {
     this->path = copyString(path);
@@ -117,23 +154,23 @@ public:
     italic = desc->italic;
     monospace = desc->monospace;
   }
-  
+
   const char* get_path() {
     return path == NULL ? "" : path;
   }
-  
+
   const char* get_psname() {
     return postscriptName == NULL ? "" : postscriptName;
   }
-  
+
   const char* get_family() {
     return family == NULL ? "" : family;
   }
-  
+
   const char* get_style() {
     return style == NULL ? "" : style;
   }
-  
+
   int get_weight() {
     switch (weight) {
     case FontWeightThin: return 1;
@@ -145,12 +182,12 @@ public:
     case FontWeightBold: return 7;
     case FontWeightUltraBold: return 8;
     case FontWeightHeavy: return 9;
-      
+
     case FontWeightUndefined: return 0;
     }
     return 0;
   }
-  
+
   int get_width() {
     switch (width) {
     case FontWidthUltraCondensed: return 1;
@@ -162,7 +199,7 @@ public:
     case FontWidthExpanded: return 7;
     case FontWidthExtraExpanded: return 8;
     case FontWidthUltraExpanded: return 9;
-      
+
     case FontWidthUndefined: return 0;
     }
     return 0;
@@ -186,6 +223,32 @@ public:
     style = NULL;
   }
 
+  bool operator==(FontDescriptor& other) {
+    if (postscriptName && !strcmp_no_case(postscriptName, other.postscriptName))
+      return false;
+
+    if (family && !strcmp_no_case(family, other.family))
+      return false;
+
+    if (style && !strcmp_no_case(style, other.style))
+      return false;
+
+    if (weight && weight != other.weight)
+      return false;
+
+    if (width && width != other.width)
+      return false;
+
+    if (italic != other.italic)
+      return false;
+
+    return true;
+  }
+
+  bool operator!=(FontDescriptor& other) {
+    return !this->operator==(other);
+  }
+
 private:
   char *copyString(const char *input) {
     if (input == NULL) {
